package ProjectTankHard.DrawningObjects;

import ProjectTankHard.DirectionType;
import ProjectTankHard.Entities.EntityTankBase;

import java.awt.*;
import java.awt.geom.RoundRectangle2D;
import java.util.Random;

public class DrawningTankBase {
    protected EntityTankBase EntityTankBase;
    private int _pictureWidth;
    private int _pictureHeight;
    protected int _startPosX = 0;
    protected int _startPosY = 0;
    private int _tankWidth = 150;
    private int _tankHeight = 100;
    protected IDraw DrawningWheels;
    public EntityTankBase EntityTankBase(){
        return EntityTankBase;
    }

    public DrawningTankBase(int speed, double weight, Color bodyColor, int width, int height){
        if(width <= _tankWidth || height <= _tankHeight)  return;

        _pictureWidth = width;
        _pictureHeight = height;

        EntityTankBase = new EntityTankBase(speed, weight, bodyColor);

        Random rand = new Random();
        int variant = rand.nextInt(0, 2 + 1);

        switch (variant) {
            case 0:
                DrawningWheels = new DrawningWheels(_tankWidth, _tankHeight, _startPosX, _startPosY, bodyColor);
                break;
            case 1:
                DrawningWheels = new DrawningWheelsOrn(_tankWidth, _tankHeight, _startPosX, _startPosY, bodyColor);
                break;
            case 2:
                DrawningWheels = new DrawningWheelsOrn2(_tankWidth, _tankHeight, _startPosX, _startPosY, bodyColor);
                break;
        }

        DrawningWheels.ChangeWheelsQuantity(rand.nextInt(4, 6 + 1));
    }

    public void SetPosition(int x, int y){
        if (EntityTankBase == null)
            return;

        _startPosX = x;
        _startPosY = y;

        if (x + _tankWidth >= _pictureWidth || y + _tankHeight >= _pictureHeight  || x < 0 || y < 0){
            _startPosX = 0;
            _startPosY = 0;
        }

        DrawningWheels.ChangeX(_startPosX);
        DrawningWheels.ChangeY(_startPosY);
    }
    public int GetPosX(){return _startPosX;}
    public int GetPosY(){return _startPosY;}
    public int GetWidth(){return _tankWidth;}
    public int GetHeight(){return _tankHeight;}
    public boolean CanMove(DirectionType direction)
    {
        if (EntityTankBase == null)
            return false;

        switch (direction)
        {
            case Left:
                return _startPosX - EntityTankBase.Step() >= 0;
            case Right:
                return _startPosX + EntityTankBase.Step() + _tankWidth< _pictureWidth;
            case Down:
                return _startPosY + EntityTankBase.Step() + _tankHeight < _pictureHeight;
            case Up:
                return _startPosY - EntityTankBase.Step() >= 0;
        };

        return false;
    }
    public void MoveTransport(DirectionType direction){
        if (!CanMove(direction) || EntityTankBase == null) return;

        switch (direction)
        {
            case Left:
                if (_startPosX - EntityTankBase.Step() >= 0)
                    _startPosX -= (int) EntityTankBase.Step();
                break;
            case Up:
                if (_startPosY - EntityTankBase.Step() >= 0)
                    _startPosY -= (int) EntityTankBase.Step();
                break;
            case Right:
                if (_startPosX + EntityTankBase.Step() + _tankWidth <= _pictureWidth)
                    _startPosX += (int) EntityTankBase.Step();
                break;
            case Down:
                if (_startPosY + EntityTankBase.Step() + _tankHeight <= _pictureHeight)
                    _startPosY += (int) EntityTankBase.Step();
                break;
        }

        DrawningWheels.ChangeX(_startPosX);
        DrawningWheels.ChangeY(_startPosY);
    }

    public void DrawTank(Graphics2D g2d) {
        if (EntityTankBase == null) return;

        g2d.setColor(EntityTankBase.BodyColor());

        // гусеница
        RoundRectangle2D caterpillars = new RoundRectangle2D.Double(_startPosX, _startPosY + 65, _tankWidth, 35, 20, 20);
        g2d.draw(caterpillars);

        // колеса
        DrawningWheels.DrawWheels(g2d);

        // башня
        g2d.setColor(EntityTankBase.BodyColor());
        g2d.fillRect(_startPosX + 45, _startPosY + 30, 60, 25);
        g2d.fillRect(_startPosX + 5, _startPosY + 55 + 1, _tankWidth - 10, 9);

    }
}
